==> fetch.php?media=download_software:mae3xx:mae3xx_rootfs_bionic_v4_1_0.tar.xz: usr/local/lib/python3.6/dist-packages/ppplib.py <==
#! /usr/bin/env python3
# PPP related library
#

import os
import serial
import subprocess
import syslog


class LN940(object):
    def __init__(self, idx=0):
        self._port = "/dev/cdc-wdm{}".format(idx)
        self._configurations = {}
        self.get_list()

    def get_list(self):
        cmdline = "qmicli -p -d {} --pdc-list-configs=software".format(self._port)
        res, output = subprocess.getstatusoutput(cmdline)

        if res != 0:
            return False

        configurations = {}

        for line in output.split('\n'):
            if line.startswith("Configuration "):
                info = {}
            elif line.startswith('\t'):
                key, *values = [x.strip() for x in line.strip().split(':')]
                value = ''.join(values)
                info[key] = value
            elif not line:
                configurations[info["Description"].upper()] = info
                info = {}

        self._configurations = configurations

    def select_firmware(self, carrier):
        if not self._configurations:
            res = self.get_list()

            if not res:
                raise IOError("cannot get carrier firmware list.")

        desc = carrier.upper()
        config = self._configurations.get(desc, None)

        if not config:
            msg = "LN940: firmware \"{}\" not found.".format(carrier)
            syslog.syslog(syslog.LOG_ERR, msg)
            return False

        if config["Status"] == "Active":
            msg = "LN940: firmware \"{}\" is already selected.".format(carrier)
            syslog.syslog(syslog.LOG_INFO, msg)
            return True

        msg = "LN940: activate firmware \"{}\".".format(carrier)
        syslog.syslog(syslog.LOG_INFO, msg)

        id = config["ID"]
        self._configurations = {}

        cmdline = "qmicli -d {} --pdc-activate-config=software,{}".format(
            self._port, id)
        res, output = subprocess.getstatusoutput(cmdline)

        if res != 0:
            msg = "LN940: activate firmware failed."
            syslog.syslog(syslog.LOG_ERR, msg)
            return False
        else:
            msg = "LN940: activate firmware succeeded."
            syslog.syslog(syslog.LOG_INFO, msg)
            return True

==> fetch.php?media=download_software:mae3xx:mae3xx_rootfs_bionic_v4_2_1.tar.xz: usr/local/lib/python3.6/dist-packages/ppplib.py <==
#! /usr/bin/env python3
# PPP related library
#

import os
import serial
import subprocess
import syslog


class LN940(object):
    def __init__(self, idx=0):
        self._port = "/dev/cdc-wdm{}".format(idx)
        self._configurations = {}
        self.get_list()

    def get_list(self):
        cmdline = "qmicli -p -d {} --pdc-list-configs=software".format(self._port)
        res, output = subprocess.getstatusoutput(cmdline)

        if res != 0:
            return False

        configurations = {}

        for line in output.split('\n'):
            if line.startswith("Configuration "):
                info = {}
            elif line.startswith('\t'):
                key, *values = [x.strip() for x in line.strip().split(':')]
                value = ''.join(values)
                info[key] = value
            elif not line:
                configurations[info["Description"].upper()] = info
                info = {}

        self._configurations = configurations

    def select_firmware(self, carrier):
        if not self._configurations:
            res = self.get_list()

            if not res:
                raise IOError("cannot get carrier firmware list.")

        desc = carrier.upper()
        config = self._configurations.get(desc, None)

        if not config:
            msg = "LN940: firmware \"{}\" not found, use GCF.".format(carrier)
            syslog.syslog(syslog.LOG_ERR, msg)
            config = self._configurations.get("GCF", None)

            if not config:
                return False

        if config["Status"] == "Active":
            msg = "LN940: firmware \"{}\" is already selected.".format(carrier)
            syslog.syslog(syslog.LOG_INFO, msg)
            return True

        msg = "LN940: activate firmware \"{}\".".format(carrier)
        syslog.syslog(syslog.LOG_INFO, msg)

        id = config["ID"]
        self._configurations = {}

        cmdline = "qmicli -d {} --pdc-activate-config=software,{}".format(
            self._port, id)
        res, output = subprocess.getstatusoutput(cmdline)

        if res != 0:
            msg = "LN940: activate firmware failed."
            syslog.syslog(syslog.LOG_ERR, msg)
            return False
        else:
            msg = "LN940: activate firmware succeeded."
            syslog.syslog(syslog.LOG_INFO, msg)
            return True


class AMP520(object):
    def __init__(self):
        self._s = serial.Serial("/dev/ttyAMP0")
        self._s.timeout = 0.2

    def get_revision(self):
        self._s.write(b"AT+CGMR\r\n")

        buf = self._s.readlines()
        if len(buf) < 2:
            return 0

        try:
            _rev = [int(x) for x in buf[-1].decode().strip().split("-")]

            if len(_rev) != 2:
                raise ValueError

            rev = _rev[0] + _rev[1] / 100
        except:
            rev = 0

        return rev

    def is_simlock_free(self):
        rev = self.get_revision()

        if rev >= 18.04:
            res = 1
        else:
            res = 0

        return res
